/*
 * FileInfo.java - stores information about entity in project
 * Copyright (C) 1999-2000 Lubos Balat
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package sk.jeeff.shed.gui;
import sk.jeeff.shed.*;
import sk.jeeff.shed.project.*;
import java.io.*;
import java.util.*;
import java.text.*;

public class FileInfo implements Serializable
{
   private String fileName;
   private String filePath;
   private boolean isDirectory;
   private String elementName;

   private int type;

   private int allocation;

   public static final int PROJECT_ROOT=0;
   public static final int FILE=1;
   public static final int DIR=2;
   public static final int SUB_FILE=3;

   public static final int ALLOCATED_FREE=0;
   public static final int ALLOCATED_BY_ME=1;
   public static final int ALLOCATED_BY_OTHER=2;

   private String owner="";

   //additional information about file
   private Date createDate = null;
   private String createdBy = "";
   private Date lockedDate = null;

   //date formator
   private SimpleDateFormat df = new SimpleDateFormat();

   public FileInfo()
   {

   }

   public FileInfo(String fileName, int type, int allocation)
   {
      this.allocation=allocation;

      if (type==DIR)
      {
         isDirectory=true;
         this.fileName = fileName.substring(fileName.lastIndexOf('/')+1);
         this.filePath = fileName;
      }
      else
      {
         isDirectory=false;
         if (fileName.indexOf('/')!=-1)
         {
            this.fileName = fileName.substring(fileName.lastIndexOf('/')+1);
            this.filePath = fileName.substring(0,fileName.lastIndexOf('/'));
         }
         else
         {
            this.fileName = fileName;
            this.filePath = "";
         }
      }
      //filePath = filepath + System.getProperty("file.separator") + name;
      //TODO: set allocation owner!!!
      if (allocation == ALLOCATED_BY_ME) setOwner(ProjectInfo.myName);
      else if (allocation == ALLOCATED_FREE) setOwner("");
      else if (allocation == ALLOCATED_BY_OTHER) setOwner("UNKNOWN_OWNER");

      this.type=type;
      filePath.replace('\\','/');
   }

   public void setType(int type) {this.type=type;}
   public int getType() {return type;}

   //get names of types (this is used in FileInfoPropertiesDialog.java)
   static String[] getTypeNames()
   {
      String types[]=new String[4];
      types[0]="PROJECT_ROOT";
      types[1]="FILE";
      types[2]="DIR";
      types[3]="ELEMENT";
      return(types);
   }

   public String getFileName()
   {
      return (fileName);
   }

   public String getFilePath()
   {
      return (filePath);
   }

   //get file with path
   public String getPathWithFile()
   {
      if (filePath.length()<1)
      {
         return (fileName);
      }
      else return (filePath+"/"+fileName);
   }

   public void setElementName(String elementName)
   {
      this.elementName=elementName;
   }

   public String getElementName()
   {
      return(elementName);
   }

   public void setCreator(String name)
   {
      this.createdBy = name;
   }

   public String getCreator()
   {
      return(createdBy);
   }

   public void setCreateDate(long timestamp)
   {
      createDate = new Date(timestamp);
   }

   public String getCreateDate()
   {
      if (createDate == null) return("");
      return(df.format(createDate));
   }

   public void setLockedDate(long timestamp)
   {
      lockedDate = new Date(timestamp);
   }

   public String getLockedDate()
   {
      if (lockedDate == null) return("");
      return(df.format(lockedDate));
   }

   public String toString()
   {
      if (type==SUB_FILE) return (elementName);
      else return(fileName);
   }

   public boolean isDirectory()
   {
      return(isDirectory);
   }

   public void setOwner(String owner)
   {
      this.owner=owner;
      if (owner.length()<1) allocation=ALLOCATED_FREE;
      else if (ProjectInfo.myName.compareToIgnoreCase(owner)==0) allocation=ALLOCATED_BY_ME;
      else allocation=ALLOCATED_BY_OTHER;

      System.out.println("owner="+owner+" project="+ProjectInfo.myName);

   }

   public String getOwner()
   {
      return(owner);
   }

   public int getAllocation()
   {
      if (owner.length()<1) allocation=ALLOCATED_FREE;
      else if (ProjectInfo.myName.compareToIgnoreCase(owner)==0) allocation=ALLOCATED_BY_ME;
      else allocation=ALLOCATED_BY_OTHER;
      return(allocation);
   }

   public boolean isFree()
   {
      if (owner.length()<1) return(true);
      else return(false);
   }

   public boolean isMine()
   {
      System.out.println("Project owner="+ProjectInfo.myName+" owner="+owner+"|");
      if (ProjectInfo.myName.compareToIgnoreCase(owner)==0) return(true);
      else return(false);
   }

   public boolean isOther()
   {
      if (owner.length()>0 && ProjectInfo.myName.compareToIgnoreCase(owner)!=0) return(true);
      else return(false);
   }

   public void print()
   {
      System.out.println("fileName="+fileName+" filePath="+filePath+" type="+type+" owner="+owner);
   }
}
