/*
 * FilesTree.java - component which show files in project
 * Copyright (C) 1999-2000 Lubos Balat
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package sk.jeeff.shed.gui;
import java.awt.*;
//import javax.swing.event;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.net.*;
import java.io.*;
import java.util.*;
import sk.jeeff.shed.gui.*;

public class FilesTree extends JPanel {
    protected DefaultMutableTreeNode rootNode;
    protected DefaultTreeModel treeModel;
    protected JTree tree;
    private Toolkit toolkit = Toolkit.getDefaultToolkit();

    private int pom1=0;

   //dirList
   char PathSep = File.separatorChar;
   String projectFiles="";

   //pre serializovanie
   Vector v_strom;

   //selected FileInfo
   private FileInfo selectedFileInfo=null;

   f_main gui;

   public FilesTree(f_main gui)
   {
        this.gui = gui;
        rootNode = new DefaultMutableTreeNode("Root Node");
        // rootNode.add(new DefaultMutableTreeNode("Root Node 2 "));

        //dirList();

        treeModel = new DefaultTreeModel(rootNode);
        treeModel.addTreeModelListener(new MyTreeModelListener());

        tree = new JTree(treeModel);
        tree.setCellRenderer(new MyRenderer());
        tree.setEditable(false);
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.putClientProperty("JTree.lineStyle", "Angled");
        tree.setShowsRootHandles(true);

        JScrollPane scrollPane = new JScrollPane(tree);
        setLayout(new GridLayout(1,0));
        add(scrollPane);

        MouseListener tree_ml = new MouseAdapter()
        {
           public void mouseClicked(MouseEvent e)
           {
              int selRow = tree.getRowForLocation(e.getX(), e.getY());
              TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
              if(selRow != -1)
              {
                 if(e.getClickCount() == 1)
                 {
                    treeSingleClick(selRow, selPath);
                 }
                 else if(e.getClickCount() == 2)
                 {
                    treeDoubleClick(selRow, selPath);
                 }
              }
           }
       };
       tree.addMouseListener(tree_ml);
    }

    private void treeSingleClick(int selRow, TreePath selPath)
    {
       DefaultMutableTreeNode selNode;
       FileInfo fileInfo;
       System.out.println("Single click");
       if (selPath == null)
       {
          selNode = rootNode;
       } else
       {
          selNode = (DefaultMutableTreeNode) (selPath.getLastPathComponent());
       }
       selectedFileInfo=(FileInfo)selNode.getUserObject();
       selectedFileInfo.print();
       setGuiStatusBar(selectedFileInfo);
    }

    private void setGuiStatusBar(FileInfo selectedFileInfo)
    {
       gui.l_locked_by.setText(selectedFileInfo.getOwner());
       gui.l_locked_date.setText(selectedFileInfo.getLockedDate());
       gui.l_created_by.setText(selectedFileInfo.getCreator());
       gui.l_created_date.setText(selectedFileInfo.getCreateDate());
    }

    private void treeDoubleClick(int selRow, TreePath selPath)
    {
       DefaultMutableTreeNode selNode;
       FileInfo fileInfo;
       System.out.println("Double click");
       if (selPath == null)
       {
          selNode = rootNode;
       } else
       {
          selNode = (DefaultMutableTreeNode) (selPath.getLastPathComponent());
       }
       fileInfo=(FileInfo)selNode.getUserObject();
       fileInfo.print();
       if (fileInfo.getType()==FileInfo.FILE || fileInfo.getType()==FileInfo.SUB_FILE) gui.editFile(gui.project.getLocalPath()+"/"+fileInfo.getPathWithFile());
    }

   //store all object in tree to vector
   public Vector storeToVector()
   {
      v_strom=new Vector();
      v_strom=addVector(rootNode, "");
      System.out.println("Objektov: "+v_strom.size());
      return(v_strom);
   }

   //add object to vector
   private Vector addVector(DefaultMutableTreeNode rodic, String odsun)
   {
      DefaultMutableTreeNode uzol;

      Vector objekty;
      objekty=new Vector();

      FileInfo fileInfo;
      uzol=rodic.getFirstLeaf();
      Enumeration deti = rodic.children();
      fileInfo=(FileInfo)rodic.getUserObject();

      objekty.add(fileInfo);

      //System.out.println(odsun+fileInfo.getFileName());
      v_strom.add(fileInfo);
      while (deti.hasMoreElements())
      {
         //uzol=rodic.getChildAt(i);
         uzol=(DefaultMutableTreeNode)deti.nextElement();
         if (uzol.isLeaf())
         {
            fileInfo=new FileInfo();
            fileInfo=(FileInfo)uzol.getUserObject();
            objekty.add(fileInfo);
            //System.out.println(odsun+fileInfo.getFileName());
         }
         else
         {
            //System.out.println("Saving: next part=");
            objekty.add(addVector(uzol, odsun+"  "));
         }
         //uzol=rodic.getNextLeaf();
      }
      return(objekty);
   }

   public void expandAll()
   {
      if (rootNode!=null) expandAll(rootNode);
   }

   private void expandAll(DefaultMutableTreeNode rodic)
   {
      DefaultMutableTreeNode uzol;
      uzol=rodic.getFirstLeaf();
      Enumeration deti = rodic.children();

      while (deti.hasMoreElements())
      {
         //uzol=rodic.getChildAt(i);
         uzol=(DefaultMutableTreeNode)deti.nextElement();
         if (uzol.isLeaf())
         {

         }
         else
         {
            tree.expandPath(new TreePath(uzol.getPath()));
            expandAll(uzol);
         }

      }
   }

   //update tree from vector
   public void readFromVector(Vector v_strom)
   {

      clear();
      System.out.println("Objektov pred read: "+v_strom.size());

      decodeVector(v_strom, null);

      System.out.println("Objektov po read: "+v_strom.size());
   }

   private void decodeVector(Vector objekty, DefaultMutableTreeNode rodic)
   {
      int i;
      DefaultMutableTreeNode node;
      FileInfo fileInfo;
      fileInfo=(FileInfo)objekty.elementAt(0);
      if (fileInfo.isDirectory()) System.out.print("[DIR]");
      System.out.println(fileInfo.getFileName());

      if (rodic==null)
      {
         node=rootNode;
         node.setUserObject(fileInfo);
      }
      else
      {
         node = new DefaultMutableTreeNode(fileInfo);
         rodic.add(node);
      }

      System.out.println("Objektov: "+objekty.size());

      for (i=1; i< objekty.size(); i++)
      {
         if (objekty.elementAt(i) instanceof FileInfo)
         {
            fileInfo = (FileInfo)objekty.elementAt(i);
            node.add(new DefaultMutableTreeNode(fileInfo));
            /*if (fileInfo.isDirectory())
            {
                System.out.print("[DIR]");
                node.add(new DefaultMutableTreeNode(fileInfo));
                //decode_vector((Vector)objekty.elementAt(i), node);
            } */
            System.out.println(fileInfo.getFileName());
         }
         else if (objekty.elementAt(i) instanceof Vector)
         {
            System.out.print("subVektor: ");
            decodeVector((Vector)objekty.elementAt(i), node);
         }
      }
   }

   //Remove all nodes except the root node
   public void clear()
   {
      rootNode.removeAllChildren();
      treeModel.reload();
   }


   //return treenode of fileinfo
   //if fileinfo is subfile, it returns parent of this fileinfo
   private DefaultMutableTreeNode getParentNode(FileInfo child)
   {
      DefaultMutableTreeNode parentNode = null;
      TreePath parentPath = tree.getSelectionPath();

      if (parentPath == null)
      {
         parentNode = rootNode;
         return null;
      }

      parentNode = (DefaultMutableTreeNode)(parentPath.getLastPathComponent());
      FileInfo fiParentNode=(FileInfo)parentNode.getUserObject();
      if (fiParentNode.getType()==FileInfo.PROJECT_ROOT || fiParentNode.getType()==FileInfo.DIR)
      {
        return null;
      }

      //get parent of sub file...
      if (fiParentNode.getType()==FileInfo.SUB_FILE)
      {
         parentNode=(DefaultMutableTreeNode)parentNode.getParent();
      }
      return (parentNode);
   }

   //return element names of current selected node separated by #
   public String getElementNames()
   {
      FileInfo child = getSelectedFileInfo();
      DefaultMutableTreeNode parentNode = getParentNode(child);

      if (parentNode == null) return(null);

      int i;
      int childCount=parentNode.getChildCount();

      if (childCount==0) return(null);

      DefaultMutableTreeNode parentChildNode;
      FileInfo parentChildFileInfo;
      String ret="";
      for (i=0; i<childCount; i++)
      {
         parentChildNode=(DefaultMutableTreeNode)parentNode.getChildAt(i);
         parentChildFileInfo=(FileInfo)parentChildNode.getUserObject();
         ret=ret+"#"+parentChildFileInfo.getElementName();
      }
      return(ret);
   }

   //return element names of current selected node separated by #
   //but only elements which is not mine...
   public String getElementNamesNotMine()
   {
      FileInfo child = getSelectedFileInfo();
      DefaultMutableTreeNode parentNode = getParentNode(child);

      if (parentNode == null) return(null);

      int i;
      int childCount=parentNode.getChildCount();

      if (childCount==0) return(null);

      DefaultMutableTreeNode parentChildNode;
      FileInfo parentChildFileInfo;
      String ret="";
      for (i=0; i<childCount; i++)
      {
         parentChildNode=(DefaultMutableTreeNode)parentNode.getChildAt(i);
         parentChildFileInfo=(FileInfo)parentChildNode.getUserObject();
         if (!parentChildFileInfo.isMine())
         {
            ret=ret+"#"+parentChildFileInfo.getElementName();
         }
      }
      return(ret);
   }

   //return element names of current selected node separated by #
   //but only elements which is mine...
   public String getMineElementNames()
   {
      FileInfo child = getSelectedFileInfo();
      DefaultMutableTreeNode parentNode = getParentNode(child);

      if (parentNode == null) return(null);

      int i;
      int childCount=parentNode.getChildCount();

      if (childCount==0) return(null);

      DefaultMutableTreeNode parentChildNode;
      FileInfo parentChildFileInfo;
      String ret="";
      for (i=0; i<childCount; i++)
      {
         parentChildNode=(DefaultMutableTreeNode)parentNode.getChildAt(i);
         parentChildFileInfo=(FileInfo)parentChildNode.getUserObject();
         if (parentChildFileInfo.isMine())
         {
            ret=ret+"#"+parentChildFileInfo.getElementName();
         }
      }
      return(ret);
   }

   //Add child to the currently selected node
   public DefaultMutableTreeNode addObject(FileInfo child)
   {
      return(addObject(child, false));
   }

   public void test()
   {
      //System.out.println("TEST");

      FileInfo finfo = new FileInfo("/Folder1/Folder1-1/file1-2.java", FileInfo.SUB_FILE, FileInfo.ALLOCATED_FREE);
      finfo.setElementName("element1");
      addToTree(finfo);
   }

   public void addToTree(FileInfo finfo)
   {
      //System.out.println("Add to tree");
      String path = finfo.getFilePath();
      String file = "";
      String element = "";
      if (finfo.getType()==FileInfo.FILE)
      {
         file = finfo.getFileName();
      }
      if (finfo.getType()==FileInfo.SUB_FILE)
      {
         file = finfo.getFileName();
         element = finfo.getElementName();
      }
      addToTree(rootNode, path, file, element, finfo);
      setGuiStatusBar(finfo);
   }

   private void addToTree(DefaultMutableTreeNode rodic, String path, String file, String element, FileInfo finfo)
   {
      //System.out.println("--- Adding to tree");
      if (rodic==null)
      {
         rodic=rootNode;
      }

      DefaultMutableTreeNode uzol;

      FileInfo fileInfo;
      uzol=rodic.getFirstLeaf();
      Enumeration deti = rodic.children();
      fileInfo=(FileInfo)rodic.getUserObject();

      StringTokenizer pt = new StringTokenizer(path, "/");
      String t_path=null;
      if (pt.hasMoreTokens()) t_path=pt.nextToken();
      //System.out.println("t_path="+t_path);

      while (deti.hasMoreElements())
      {
         //uzol=rodic.getChildAt(i);
         uzol=(DefaultMutableTreeNode)deti.nextElement();
         if (t_path != null)
         {
            if (!uzol.isLeaf())
            {
               fileInfo=(FileInfo)uzol.getUserObject();
               //System.out.println("fileName="+fileInfo.getFileName());

               if (fileInfo.getFileName().compareTo(t_path)==0)
               {
                  //System.out.println("NASIEL SOM:"+fileInfo.getFileName());
                  if (pt.hasMoreTokens()) path = pt.nextToken("\0");
                  else path="";
                  //System.out.println("ZVYSNA_CESTA="+path);
                  addToTree(uzol, path, file, element, finfo);
                  return;
               }
            }
         }
         //ak je zadany element, najdi k nemu este subor
         else if (file.length()>0)
         {
            if (!uzol.isLeaf())
            {
               fileInfo=(FileInfo)uzol.getUserObject();
               //System.out.println("fileName="+fileInfo.getFileName());

               if (fileInfo.getFileName().compareTo(file)==0)
               {
                  //System.out.println("NASIEL SOM FILE:"+fileInfo.getFileName());
                  //ak je este element, pokracuj
                  if (element.length()>0)
                  {
                     addToTree(uzol, "", "", element, finfo);
                  }
                  else
                  {
                     //System.out.println("FINISH FILE:"+fileInfo.getFileName());
                     uzol.setUserObject(finfo);
                     treeModel.nodeChanged(uzol);
                  }
                  return;
               }
            }
         }
         else
         {
            //toto je uz element
            //akus sa pozriet ci uz existuje
            if (uzol.isLeaf())
            {
               fileInfo=(FileInfo)uzol.getUserObject();
               //System.out.println("fileName="+fileInfo.getFileName());

               if (fileInfo.getElementName().compareTo(element)==0)
               {
                  //System.out.println("FINISH MAM ELEMENT:"+fileInfo.getFileName());
                  uzol.setUserObject(finfo);
                  treeModel.nodeChanged(uzol);
                  return;
               }
            }
         }
         //uzol=rodic.getNextLeaf();
      }
      //nema potomkov, alebo nenasiel pozadovany uzol
      //treba insertnut
      //System.out.println(" NEED INSERT");

      if (t_path != null)
      {
         //insert cesty
         FileInfo fi_parent = (FileInfo)rodic.getUserObject();
         FileInfo fi_path = new FileInfo(fi_parent.getFilePath()+"/"+t_path, FileInfo.DIR, FileInfo.ALLOCATED_FREE);
         //System.out.println("INSERTING DIR:"+fi_path.getFilePath());
         DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(fi_path);
         treeModel.insertNodeInto(childNode, rodic, rodic.getChildCount());
         if (pt.hasMoreTokens()) path = pt.nextToken("\0");
         else path="";
         //System.out.println("ZVYSNA_CESTA="+path);
         addToTree(childNode, path, file, element, finfo);
         return;
      }
      else if (file.length()>0)
      {
         //insert suboru
         FileInfo fi_parent = (FileInfo)rodic.getUserObject();
         FileInfo fi_file = new FileInfo(fi_parent.getFilePath()+"/"+file, FileInfo.FILE, FileInfo.ALLOCATED_FREE);
         //System.out.println("INSERTING FILE:"+fi_file.getPathWithFile());
         DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(fi_file);
         treeModel.insertNodeInto(childNode, rodic, rodic.getChildCount());
         if (element.length() > 0)
         {
            //mame este element
            addToTree(childNode, "", "", element, finfo);
         }

         return;
      }
      else if (element.length() > 0)
      {
         //insert elementu
         //tu mozem rovno strcit finfo
         DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(finfo);
         treeModel.insertNodeInto(childNode, rodic, rodic.getChildCount());
         tree.scrollPathToVisible(new TreePath(childNode.getPath()));
         tree.setSelectionPath(new TreePath(childNode.getPath()));
      }
      return;
   }

   public DefaultMutableTreeNode addObject(FileInfo child, boolean shouldBeSelected)
   {

      DefaultMutableTreeNode parentNode = null;
      TreePath parentPath = tree.getSelectionPath();

      if (parentPath == null)
      {
         parentNode = rootNode;
         return addObject(parentNode, child, shouldBeSelected, -1);
      }
      else
      {
         parentNode = (DefaultMutableTreeNode)(parentPath.getLastPathComponent());
         FileInfo fiParentNode=(FileInfo)parentNode.getUserObject();
         if (fiParentNode.getType()==FileInfo.PROJECT_ROOT || fiParentNode.getType()==FileInfo.DIR || fiParentNode.getType()==FileInfo.FILE)
         {
           // System.out.println("--- Adding ---");
            return addObject(parentNode, child, shouldBeSelected, -1);
         }
         if (fiParentNode.getType()==FileInfo.SUB_FILE)
         {
            //System.out.println("--- Adding after SUB-FILE ---");

            DefaultMutableTreeNode parentParentNode=(DefaultMutableTreeNode)parentNode.getParent();
            FileInfo fParentParent=(FileInfo)parentParentNode.getUserObject();

            System.out.println(fParentParent.getFileName());

            int i;
            int index=0;
            int childCount=parentParentNode.getChildCount();
            DefaultMutableTreeNode parentChildNode;
            FileInfo parentChildFileInfo;
            for (i=0; i<childCount; i++)
            {
               parentChildNode=(DefaultMutableTreeNode)parentParentNode.getChildAt(i);
               parentChildFileInfo=(FileInfo)parentChildNode.getUserObject();
               if (parentChildFileInfo.getElementName().equals(selectedFileInfo.getElementName()))
               {
                  index=i;
                  i=childCount+1;
               }
            }

            System.out.println("INDEX="+index);

            return addObject(parentParentNode, child, shouldBeSelected, index+1);
         }
      }
      return (null);
   }

   public DefaultMutableTreeNode addObject(DefaultMutableTreeNode parent, FileInfo child, boolean shouldBeSelected, int insertIndex)
   {
      DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child);

      if (parent == null)
      {
         parent = rootNode;
      }

      System.out.println("child count="+parent.getChildCount());
      if (insertIndex==-1)
      {
         insertIndex=parent.getChildCount();
      }

      treeModel.insertNodeInto(childNode, parent, insertIndex);

      // Make sure the user can see the lovely new node.
      tree.scrollPathToVisible(new TreePath(childNode.getPath()));
      if (shouldBeSelected)
      {
         tree.setSelectionPath(new TreePath(childNode.getPath()));
      }
      return childNode;
   }



    /** Remove the currently selected node. */
   public void removeCurrentNode()
   {
      TreePath currentSelection = tree.getSelectionPath();
      if (currentSelection != null)
      {
         DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) (currentSelection.getLastPathComponent());
         MutableTreeNode parent = (MutableTreeNode)(currentNode.getParent());
         if (parent != null)
         {
            treeModel.removeNodeFromParent(currentNode);
            return;
         }
      }
      // Either there was no selection, or the root was selected.
      toolkit.beep();
   }

   //set object in current selected node
   public void setCurrentNode(FileInfo fileInfo)
   {
      TreePath currentSelection = tree.getSelectionPath();
      if (currentSelection != null)
      {
         DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) (currentSelection.getLastPathComponent());
         currentNode.setUserObject(fileInfo);
      }
      else toolkit.beep();
   }

   public FileInfo getSelectedFileInfo()
   {
      return(selectedFileInfo);
   }

   public FileInfo getRootFileInfo()
   {
      return((FileInfo)rootNode.getUserObject());
   }





// ****************** OTHER CLASSES *******************************
    class MyTreeModelListener implements TreeModelListener
    {
        public void treeNodesChanged(TreeModelEvent e)
        {
            //FileInfo fileInfo;
            DefaultMutableTreeNode node;
            node = (DefaultMutableTreeNode)
                     (e.getTreePath().getLastPathComponent());

            /*
             * If the event lists children, then the changed
             * node is the child of the node we've already
             * gotten.  Otherwise, the changed node and the
             * specified node are the same.
             */
            try
            {
                int index = e.getChildIndices()[0];
                node = (DefaultMutableTreeNode)
                       (node.getChildAt(index));
                //fileInfo=(FileInfo)node.getUserObject();
                //System.out.println(fileInfo.fileName+" path="+fileInfo.filePath);
            } catch (NullPointerException exc) {}

            System.out.println("The user has finished editing the node.");
            System.out.println("New value: " + node.getUserObject());
        }
        public void treeNodesInserted(TreeModelEvent e) {
        }
        public void treeNodesRemoved(TreeModelEvent e) {
        }
        public void treeStructureChanged(TreeModelEvent e) {
        }
    }

    //this render tree object
    private class MyRenderer extends DefaultTreeCellRenderer {
        ImageIcon tutorialIcon;
        ImageIcon leftIcon;
        ImageIcon i_doc[]=new ImageIcon[3];
        ImageIcon i_txt[]=new ImageIcon[3];
        ImageIcon i_dir;

        FileInfo fileInfo;

        int comp=0;

        private int type;
        private int imgIndex;

        public MyRenderer()
        {
            tutorialIcon = new ImageIcon("images/middle.gif");
            leftIcon = new ImageIcon("images/left.gif");
            //doc[]=new ImageIcon[3];
            i_doc[0]=new ImageIcon("images/doc-0.gif");
            i_doc[1]=new ImageIcon("images/doc-1.gif");
            i_doc[2]=new ImageIcon("images/doc-2.gif");

            i_txt[0]=new ImageIcon("images/txt-0.gif");
            i_txt[1]=new ImageIcon("images/txt-1.gif");
            i_txt[2]=new ImageIcon("images/txt-2.gif");

            i_dir=new ImageIcon("images/dir.gif");

            System.out.println("Icon: width="+i_dir.getIconWidth());
        }

        public Component getTreeCellRendererComponent
                           (JTree tree,
                            Object value,
                            boolean sel,
                            boolean expanded,
                            boolean leaf,
                            int row,
                            boolean hasFocus)
        {

            super.getTreeCellRendererComponent(
                            tree, value, sel,
                            expanded, leaf, row,
                            hasFocus);
            //if (leaf && isTutorialBook(value)) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
            //System.out.println("leaf="+leaf);
            if (node.getUserObject() instanceof FileInfo)
            {
               fileInfo = (FileInfo)(node.getUserObject());
               type = fileInfo.getType();

               //set selected file info
               if (sel==true)
               {
                  selectedFileInfo=(FileInfo)node.getUserObject();
                  selectedFileInfo.print();
               }

               //System.out.println("type="+type);
               /*if (leaf)
               {
                  //comp=1-comp;
                  if (sel) setIcon(tutorialIcon);
                  else setIcon(leftIcon);
               } */
               if (fileInfo.isFree()) imgIndex=0;
               else if (fileInfo.isOther()) imgIndex=1;
               else if (fileInfo.isMine()) imgIndex=2;

               if (type==FileInfo.FILE) setIcon(i_doc[imgIndex]);
               else if (type==FileInfo.SUB_FILE) setIcon(i_txt[imgIndex]);
               else if (type==FileInfo.DIR) setIcon(i_dir);
               else if (type==FileInfo.PROJECT_ROOT) setIcon(i_doc[imgIndex]);
            }
            return this;
        }
    }
}
