/*
 * ElementParser.java - implements all operation with elements
 * Copyright (C) 1999-2000 Lubos Balat
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package sk.jeeff.shed.parser;

import java.io.*;
import java.util.*;

import sk.jeeff.shed.*;

public class ElementParser
{
   private Vector lines=new Vector(100,50);
   private String filename;
   protected String elementStringBegin="";
   //protected String elementStringEnd="";
   protected String elementComment="";
   protected String elementOwner="";
   //protected String elementAllocBegin="";
   //protected String elementAllocEnd="";
   //protected String elementCommentBegin="";
   //protected String elementCommentEnd="";

   private int elementStartLineNum=0;
   private int elementEndLineNum=0;

   //parser types pointers
   static final int JAVA_PARSER=0;
   static final int C_PARSER=1;
   static final int DEFAULT_PARSER=JAVA_PARSER;

   //set parser type
   public void setParserType(String fileName)
   {
      fileName=fileName.toLowerCase();
      int index=fileName.lastIndexOf('.');
      if (index==-1 || index==fileName.length())
      {
         //UNKNOWN TYPE
         setParserType(DEFAULT_PARSER);
         return;
      }
      int type=DEFAULT_PARSER;
      String extension=fileName.substring(index);
      if (extension.equals(".java")) type=JAVA_PARSER;
      else if (extension.equals(".c") || extension.equals(".cpp")) type=C_PARSER;

      setParserType(type);
   }

   public void setParserType(int type)
   {
      if (type==JAVA_PARSER || type==C_PARSER)
      {
         //Java parser
         elementStringBegin="ELEMENT BEGIN: name=";
         //elementStringEnd="ELEMENT END: name=";
         elementComment="// ------ ";
         elementOwner="Owner: ";
         //elementAllocBegin="/*";
         //elementAllocEnd="*/";
         //elementCommentBegin="/*";
         //elementCommentEnd="*/";
      }
   }

   public String getElementStringBegin() {return (elementStringBegin);}
   //public String getElementStringEnd() {return (elementStringEnd);}
   public String getElementComment() {return (elementComment);}
   public String getElementOwner() {return (elementOwner);}
   //public String getElementAllocBegin() {return (elementAllocBegin);}
   //public String getElementAllocEnd() {return (elementAllocEnd);}
   //public String getElementCommentBegin() {return (elementCommentBegin);}
   //public String getElementCommentEnd() {return (elementCommentEnd);}

   public void readFile(String filename)
   {
      readFile(filename, "");
   }


   //elementNames is in format elementName1#elementName2#elementName3...
   public void readFile(String filename, String elementNames)
   {
      this.filename=filename;
      setParserType(filename);

      try
      {
         FileInputStream is = new FileInputStream(ShedTools.convertPathSeparatorToSystem(filename));
         BufferedReader br = new BufferedReader(new InputStreamReader(is));
         String line;

         System.out.println("PARSER: Reading file:"+filename);

         int i;
         int elementIndex=0;
         while ((line=br.readLine())!=null)
         {
            lines.add(line);
         }
         System.out.println("PARSER: File readed. lines:"+lines.size());

         br.close();
         is.close();

         if (elementNames.length()>0) updateElementNames(elementNames);
      }
      catch (Exception ex)
      {
         System.out.println("PARSER: Error: can't read file: "+ex.getMessage());
      }
   }

   private void updateElementNames(String elementNames)
   {
      StringTokenizer st_names=new StringTokenizer(elementNames, "#");
      Vector v_names=new Vector(st_names.countTokens()+2, 2);

      while (st_names.hasMoreTokens()) v_names.add(st_names.nextToken());

      String elementName;

      Vector new_lines=new Vector(lines.size(), 10);

      int i;
      int j;
      int elementLineStart;
      int elementLineEnd;
      for (i=0; i<v_names.size(); i++)
      {
         elementName=(String)v_names.elementAt(i);
         elementLineStart=findElementLineStart(elementName);
         //if element doesn't allready exist create it
         if (elementLineStart==-1)
         {
            new_lines.add(elementComment + elementStringBegin + elementName);
            new_lines.add(elementComment + elementOwner + "unknown");
            new_lines.add("");
            //new_lines.add(elementComment + elementStringEnd + elementName);
         }
         else
         {
            elementLineEnd=findElementLineEnd(elementLineStart);
            for (j=elementLineStart; j<=elementLineEnd; j++)
            {
               new_lines.add((String)lines.elementAt(j));
            }

         }
      }

      lines=new_lines;
   }

   //try to find element name in line.
   //returns elementName if line contains element declaration or null otherwise
   private String getElementNameFromLine(String line)
   {
      int strBeginLength=elementStringBegin.length();
      String elementName=null;
      if (line.length()>=strBeginLength && line.indexOf(elementStringBegin)!=-1)
      {
         elementName=line.substring(line.indexOf(elementStringBegin)+strBeginLength, line.length());
         if (elementName.charAt(elementName.length()-1)=='?') elementName=elementName.substring(0, elementName.length()-1);
         System.out.println("Found elementName="+elementName);
      }
      return(elementName);
   }

   private int findElementLineStart(String elementName)
   {
      int startLineNum=-1;
      String line;
      String strBegin=elementStringBegin+elementName.trim();
      strBegin=strBegin.toLowerCase();
      System.out.println("strBegin="+strBegin);

      int strBeginLength=strBegin.length();

      for (int i=0; i<lines.size(); i++)
      {
         line=(String)lines.elementAt(i);
         line=line.trim().toLowerCase();
         System.out.println("LINE="+line);
         if (line.length()>=strBeginLength && line.indexOf(strBegin)!=-1)
         {
            startLineNum=i;
            //i=lines.size();
            System.out.println("i="+i);
            break;
         }
      }

      System.out.println("startLineNum="+startLineNum);
      return (startLineNum);
   }

   /*public int findElementLineEnd(int elementStartLine)
   {
      return (findElementLineEnd(elementStartLine, true));
   } */

   public int findElementLineEnd(int elementStartLine) //, boolean withBlankLines)
   {
      String line;
      String strEnd=elementComment+elementStringBegin.toLowerCase();
      int strEndLength=strEnd.length();

      //posledny riadok elementu
      int endLineNum=elementStartLine;
      //posledny riadok elementu za ktorym su uz len prazdne riadky
      int endLineNumWithoutBlankLines=endLineNum;

      for (int i=elementStartLine; i<lines.size(); i++)
      {
         line=(String)lines.elementAt(i);
         line=line.trim().toLowerCase();

         endLineNum=i;
         if (line.length()>=strEndLength  && line.indexOf(strEnd)!=-1)
         {
            endLineNum=i-1;
            break;
         }
         //if (line.length()>0) endLineNumWithoutBlankLines=i;
      }

      //if (withBlankLines) return(endLineNum);
      //else return(endLineNumWithoutBlankLines);
      System.out.println("FIND element END: start="+elementStartLine+" END="+endLineNum+" TOTAL="+lines.size());
      return(endLineNum);
   }

   //write lines before element elementName to data stream dos
   public void writeBeforeElement(DataOutputStream dos,String elementName)
   {
      elementStartLineNum=findElementLineStart(elementName);
      //if element doesn't exist, append it to the end
      if (elementStartLineNum==-1) elementStartLineNum=lines.size();

      System.out.println("Element "+elementName+" Begin Line="+elementStartLineNum);
      writeFile(dos, 0, elementStartLineNum-1);
      System.out.println("BEFORE END");
   }

   //write lines after line elementEndLineNum (global) to data stream dos
   public void writeAfterElement(DataOutputStream dos)
   {
      elementEndLineNum=findElementLineEnd(elementStartLineNum+1);
      System.out.println("Element End Line="+elementEndLineNum);

      writeFile(dos, elementEndLineNum+1, lines.size());
   }

   //write all lines to data stream dos
   public void writeFile(DataOutputStream dos)
   {
      writeFile(dos, 0, lines.size());
   }

   //write all lines to file opened with readFile()
   public void writeFile()
   {
      String line;

      try
      {
         FileOutputStream os = new FileOutputStream(ShedTools.convertPathSeparatorToSystem(filename));
         BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(os));
         int size=lines.size();
         for (int i=0; i<size; i++)
         {
            line=(String)lines.elementAt(i);
            bw.write(line);
            bw.newLine();
         }

         bw.close();
         os.close();
      }
      catch (Exception ex)
      {

      }

   }

   //write lines from line fromLine to line toLine to data stream dos
   public void writeFile(DataOutputStream dos, int fromLine, int toLine)
   {
      String line;
      try
      {
         for (int i=fromLine; i<=toLine; i++)
         {
            line=(String)lines.elementAt(i);
            writeLine(dos, line);
         }

      }
      catch (Exception ex)
      {

      }
   }

   //write one line to data stream dos
   private void writeLine(DataOutputStream dos,String line) throws IOException
   {
      int j;
      int len=line.length();
      System.out.println("LINE="+line);
      for (j=0; j<len; j++) dos.writeByte((byte)line.charAt(j));
      writeNewLine(dos);
   }

   //write element elementName to file path
   public int writeElementLines(String path, String elementName) throws IOException, FileNotFoundException
   {
      int elementStartLine;
      int elementEndLine;

      elementStartLine = findElementLineStart(elementName);
      //if element doesn't exist, append it to the end
      if (elementStartLine==-1) elementStartLine=lines.size();
      elementStartLine+=2;

      elementEndLine = findElementLineEnd(elementStartLine)-1;

      FileOutputStream os=new FileOutputStream(path);
      BufferedWriter bw=new BufferedWriter(new OutputStreamWriter(os));

      int i;
      for (i=elementStartLine; i<=elementEndLine; i++)
      {
         bw.write((String)lines.elementAt(i));
         bw.newLine();
      }

      bw.flush();
      bw.close();
      os.close();
      return(1);
   }

   public void createNewElementAfter(String afterElementName, String elementName, String owner)
   {
      System.out.println("PARSER: Creating new element after: "+afterElementName);
      int elementStartLine=findElementLineStart(afterElementName);
      //if element doesn't exist, put it on begining
      int elementEndLine;
      if (elementStartLine==-1)
      {
         elementEndLine=-1;
      }
      else
      {
         elementEndLine=findElementLineEnd(elementStartLine);
      }

      try
      {
         FileOutputStream os = new FileOutputStream(filename);
         DataOutputStream dos = new DataOutputStream(os);

         if (elementStartLine != -1) writeFile(dos, 0, elementEndLine);
         //----writeFile(dos, 0, elementEndIndex);

         //writeLine(dos, elementStringBegin+elementName);
         //writeLine(dos, elementStringEnd);
         writeHeaderBegin(dos, elementName, owner);
         //writeHeaderEnd(dos, false);

         if (elementStartLine != -1) writeFile(dos,elementEndLine+1, lines.size());
         else writeFile(dos, 0, lines.size());
         dos.close();
         os.close();
      }
      catch (Exception ex)
      {
         System.out.println("Can't create new element: "+elementName+" in file: "+filename);
      }

   }

   //write start header of element
   public void writeHeaderBegin(DataOutputStream dos, String elementName, String owner) throws IOException
   {
      String line="";
      /*if (allocated)
      {
         line=elementComment + elementStringBegin + elementName;
      }
      else
      {
         line=elementComment + elementStringBegin + elementName;
      } */
      writeLine(dos, elementComment + elementStringBegin + elementName);
      writeLine(dos, elementComment + elementOwner + owner);
   }

   //write end header of element
   /*public void writeHeaderEnd(DataOutputStream dos, boolean allocated) throws IOException
   {
      String line="";
      if (allocated)
      {
         //line=elementComment + elementStringEnd;
      }
      else
      {
         //line=elementComment + elementStringEnd;
      }
      writeLine(dos, " ");
      //writeLine(dos, line);
   }*/

   //write new line
   public void writeNewLine(DataOutputStream dos) throws IOException
   {
      dos.writeByte(13);
      dos.writeByte(10);
   }

   //allocate element
   public void allocElement(String elementName, boolean allocation, String owner)
   {
      int elementStartLine=findElementLineStart(elementName);
      //if element doesn't exist, append it to the begining
      int elementLineEnd;
      if (elementStartLine==-1)
      {
         elementLineEnd=-1;
      }
      else
      {
         elementLineEnd=findElementLineEnd(elementStartLine);
      }
      int elementLineEndWithoutBlankLines=findElementLineEnd(elementStartLine); //, false);
      try
      {
         FileOutputStream os = new FileOutputStream(filename);
         DataOutputStream dos = new DataOutputStream(os);

         if (elementStartLine != -1) writeFile(dos, 0, elementStartLine-1);
         writeHeaderBegin(dos, elementName, owner);
         //writeFile(dos, elementStartLine+2, elementLineEndWithoutBlankLines);
         //writeHeaderEnd(dos, allocation);
         if (elementStartLine != -1) writeFile(dos, elementStartLine+2, lines.size());
         else writeFile(dos, 0, lines.size());
         dos.flush();
         dos.close();
         os.close();
      }
      catch (Exception ex)
      {
         //TODO: write some text
      }
   }

   public void allocElement(String elementName, String owner)
   {
      allocElement(elementName, true, owner);
   }

   public void unAllocElement(String elementName)
   {
      allocElement(elementName, false, "nobody");
   }

//--------------AUTOMATIC PARSE BY METHOD---------------------------------------

   //current parsed line
   private int mp_currentLine;
   private Vector mp_elementNames=new Vector();

   //write begin of file as HEADER element
   private void mp_writeHeaderElement(DataOutputStream dos) throws IOException
   {
      writeHeaderBegin(dos, "Declaration", "unknown");
      mp_elementNames.add("Declaration");

      mp_currentLine=0;
      //write all before first {
      while (((String)lines.elementAt(mp_currentLine)).indexOf('{')==-1)
      {
         writeLine(dos, (String)lines.elementAt(mp_currentLine));
         mp_currentLine++;
      }

      //search for next method
      int nextMethodLineStart=mp_searchForNextMethod(mp_currentLine+1);

      int i;
      for(i=mp_currentLine; i<nextMethodLineStart; i++)
      {
         writeLine(dos, (String)lines.elementAt(i));
      }

      mp_currentLine=i;

      //writeHeaderEnd(dos, true);
   }

   //search for { and then find backward line with method declaration and (
   private int mp_searchForNextMethod(int startLine)
   {
      String line;
      int currentLine=startLine;

      //search for {
      while (((String)lines.elementAt(currentLine)).indexOf('{')==-1)
      {
         currentLine++;
      }

      int backLineStart=currentLine;

      //search backward for (
      for (currentLine=backLineStart; currentLine>=startLine; currentLine--)
      {
         if (((String)lines.elementAt(currentLine)).indexOf('(')!=-1)
         {
            break;
         }
      }

      return(currentLine);
   }

   private int mp_writeNextElement(DataOutputStream dos) throws IOException
   {
      //line which we start parse...
      int mp_startLine=mp_currentLine;

      String methodName=mp_searchForMethodName(mp_currentLine);

      if (methodName.equals("")) return(-1);

      writeHeaderBegin(dos, methodName, "unknown");
      mp_elementNames.add(methodName);

      int mp_endLine=mp_serachForEndOfMethod(mp_startLine);

      int i;
      for(i=mp_startLine; i<=mp_endLine; i++)
      {
         writeLine(dos, (String)lines.elementAt(i));
      }

      //writeHeaderEnd(dos, true);

      mp_currentLine=mp_endLine+1;
      return(1);
   }

   private int mp_serachForEndOfMethod(int startLine)
   {
      int bracketCount=0;
      int currentLine=startLine;

      //search for first {
      while (((String)lines.elementAt(currentLine)).indexOf('{')==-1)
      {
         currentLine++;
      }

      bracketCount++;
      int posInLine=((String)lines.elementAt(currentLine)).indexOf('{');
      posInLine++;
      int currentPosInLine;
      int lineLength;
      String line;

      currentLine--;
      while (bracketCount>0)
      {
         currentLine++;
         line=(String)lines.elementAt(currentLine);
         lineLength=line.length();
         for (currentPosInLine=posInLine; currentPosInLine<lineLength; currentPosInLine++)
         {
            if (line.charAt(currentPosInLine)=='{') bracketCount++;
            if (line.charAt(currentPosInLine)=='}') bracketCount--;
            if (bracketCount==0) break;
         }
         posInLine=0;
      }

      return (currentLine);
   }

   private String mp_searchForMethodName(int inLine)
   {
      if (inLine>=lines.size()) return ("");

      String line=(String)lines.elementAt(inLine);
      //search for (
      int bracket=line.indexOf('(');

      if (bracket==-1) return(mp_searchForMethodName(inLine+1));

      line = line.substring(0, bracket);
      line = line.trim();
      int space=line.lastIndexOf(' ');

      if (space==-1) space=0;

      line = line.substring(space);
      line = line.trim();
      return (line);
   }

   public Vector parseByMethodName(String fileName)
   {
      readFile(fileName);

      try
      {
         FileOutputStream os = new FileOutputStream(ShedTools.convertPathSeparatorToSystem(fileName));
         DataOutputStream dos = new DataOutputStream(os);

         mp_elementNames.clear();

         mp_writeHeaderElement(dos);

         while (mp_writeNextElement(dos)!=-1);


         //write last element
         int i;
         int size=lines.size();
         writeHeaderBegin(dos, "End of file", "unknown");
         mp_elementNames.add("End of file");
         for (i=mp_currentLine; i<size; i++)
         {
            writeLine(dos, (String)lines.elementAt(i));
         }
         //writeHeaderEnd(dos, true);

         dos.close();
         os.close();

         return(mp_elementNames);
      }
      catch (Exception ex)
      {
         System.out.println("Can't parse file "+fileName+" by Method");
         return(null);
      }
   }


}