/*
 * shedServer.java - server main class
 * Copyright (C) 1999-2000 Lubos Balat
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package sk.jeeff.shed.server;

import java.awt.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;
import java.sql.*;

public class shedServer implements WindowListener
{
   static final int MaxConnects=20;
   static Client clients[] = new Client[MaxConnects];
   static ServerSocket server = null;
   static ServerSocket dserver = null;
   static int active_connects = 0;
   static Socket socket = null;
   static Socket dsocket = null;

   //constants
   static final String PROJECTS_DIR="projects";
   static final int PORT=2525;

	int DEBUG=1;

   private static String lockAppend=".LOCK.";

   public void windowClosed(WindowEvent e)
   {
      dbg("Window closed");
   }

   public void windowOpened(WindowEvent e) {}
   public void windowClosing(WindowEvent e) {}
   public void windowIconified(WindowEvent e) {}
   public void windowDeiconified(WindowEvent e) {}
   public void windowActivated(WindowEvent e) {}
   public void windowDeactivated(WindowEvent e) {}


   //notify all connected users about connected users
   public static void notifyRoom()
   {
      StringBuffer people = new StringBuffer("/people");

      for (int i=0; i<clients.length;i++)
         if (clients[i] != null)
            people.append("#" + i + "-" + clients[i].getUserName() );
      sendClients(people);
   }

   //send message msg to all clients
   public static void sendClients(StringBuffer msg)
   {
      System.out.println("Sending:" + msg);
      for (int i=0; i<clients.length;i++)
      {
         if (clients[i] != null) {
            System.out.println("Sent to:" + clients[i].getUserName());
            clients[i].send(msg);
        }
      }
   }

   public static void sendClients(String msg)
   {
      System.out.println("Sending:" + msg);
      for (int i=0; i<clients.length;i++)
      {
         if (clients[i] != null) {
            System.out.println("Sent to:" + clients[i].getUserName());
            clients[i].send(msg);
        }
      }
   }

   //send message msg to client with id i
   public static void sendClient(StringBuffer msg,int i)
   {
      System.out.println("Sending:" + msg);
      if (clients[i] != null)
      {
         System.out.println("Sent to:" + clients[i].getUserName());
         clients[i].send(msg);
      }
   }

   public static void sendClient(String msg,int i)
   {
      System.out.println("Sending:" + msg);
      if (clients[i] != null)
      {
         System.out.println("Sent to:" + clients[i].getUserName());
         clients[i].send(msg);
      }
   }

   //close all connections
   public static void closeAll() {
      System.out.println("Closing");
      for (int i=0; i<clients.length;i++) {
         if (clients[i] != null) {
            try {
               clients[i].socket.close();
            } catch (IOException e) {
               System.out.println("Error: " + e);
            }
         }
      }
   }

   //check if someone use nick name
   //return: true if nick name allready someone use
   //        false otherwise
   public static boolean checkName(Client c)
   {
	   //System.out.println("ID=" + c.ID + " name=" + c.name);
      for (int i=0; i<clients.length;i++)
      {
         if (clients[i] != null)
         {
		      //System.out.println("clients=" + clients[i].name);
            if ((c.ID != i) && clients[i].getUserName().equals(c.getUserName()))
            {
	            System.out.println("Name exists!!!");
               return(false);
            }
         }
      }
      return(true);
   }

   //disconnect client with id ID
   public static synchronized void disconnect(int ID, String reason)
   {
      clients[ID].send(new StringBuffer("/quit#"+reason));
      try {
         clients[ID].socket.close();
      } catch (IOException e) {
         System.out.println("Error: " + e);
      } finally
      {
         clients[ID] = null;
         socket = null;
         active_connects--;
      }
   }

   //main method
   public static void main(String args[])
   {
      try
      {
         server = new ServerSocket(shedServer.PORT);
		   dserver = new ServerSocket(shedServer.PORT+1);
		   System.out.println("Server running, waiting for clients....");
      } catch (IOException e)
      {
         System.out.println("Error: " + e);
      }

      while(true)
      {
         if (active_connects < MaxConnects)
         {
            try
            {
               socket = server.accept();
			      dsocket = dserver.accept();
            } catch (IOException e) {
               System.out.println("Error: " + e);
            }
	         active_connects++;
            for (int i=0; i<shedServer.clients.length; i++)
            {
               if (clients[i] == null)
               {
                  clients[i] = new Client(socket, dsocket, i);
		            if (active_connects < MaxConnects)
		            {
                     //check password
                     if (clients[i].checkPassword() == false)
                     {
                        disconnect(i, "Wrong username or password");
                        break;
                     }
                     if (checkName(shedServer.clients[i]))
                     {
                        shedServer.clients[i].start();
                        notifyRoom();
                     }
                     else
                     {
                        disconnect(i, "SORRY Sorry name already taken.");
                     }
                     break;
                  } else
                  {
                     disconnect(i, "SORRY Sorry server is full, try connecting later.");
			            break;
		            }
	            }
            }
         } else
         {
            try
            {
               Thread.sleep(200);
            } catch (InterruptedException e) { }
         }
      }
   }

   public void err(String text)
   {
		System.err.println(text);
   }

   public void dbg(String text)
   {
		if (DEBUG==1) System.out.println(text);
   }

   //allocate file
   static synchronized boolean allocFile(String path)
   {
      File f=new File(path+lockAppend);
      try
      {
         if (!f.exists() && f.createNewFile())
         {
            System.out.println("NEW lock"+path);
            return (true);
         }
         else return(false);
      }
      catch (Exception ex)
      {

      }
      return (false);
   }

   //unallocate file
   static synchronized boolean unAllocFile(String path)
   {
      File f=new File(path+lockAppend);
      if (f.exists())
      {
         f.delete();
         System.out.println("NEW unLock: "+path);
      }
      return(true);
   }

   //allocate element
   static synchronized boolean allocElement(int element_id, int userId, Connection dbcon)
   {
      boolean ret = false;
      //first check if element is unallocated
      try
      {
         Statement st = dbcon.createStatement();
         ResultSet rs = st.executeQuery("SELECT locked_by FROM elements WHERE element_id="+element_id);
         if (rs.next())
         {
            if (rs.getInt(1)==0)
            {
               ret=true;
               java.util.Date now = new java.util.Date();
               long i_now = (long) (now.getTime() / 1000);
               st.execute("UPDATE elements SET locked_by="+userId+", locked_date="+i_now+" WHERE element_id="+element_id);
            }
         }
         rs.close();
         st.close();
      }
      catch (Exception ex)
      {
         System.err.println("ERR: allocElement:"+ex.getMessage());
      }
      System.out.println("Allocating element: "+element_id+" ret="+ret);
      return(ret);
   }

   //unallocate element
   static synchronized boolean unAllocElement(int element_id, int userId, Connection dbcon)
   {
      boolean ret = false;
      //first check if element is unallocated
      try
      {
         Statement st = dbcon.createStatement();
         ResultSet rs = st.executeQuery("SELECT locked_by FROM elements WHERE element_id="+element_id);
         if (rs.next())
         {
            if (rs.getInt(1)==userId)
            {
               ret=true;
               st.execute("UPDATE elements SET locked_by=0 WHERE element_id="+element_id);
            }
         }
         rs.close();
         st.close();
      }
      catch (Exception ex)
      {
         System.err.println("ERR: unAllocElement:"+ex.getMessage());
      }
      return(ret);
   }

   //create project
   static synchronized boolean newProject(StringTokenizer stok, Connection dbcon)
   {
      boolean ret = false;
      try
      {
         String projectName = stok.nextToken();
         String projectLeader = stok.nextToken();

         //first check if project already doesn't exist
         Statement st = dbcon.createStatement();
         ResultSet rs = st.executeQuery("SELECT project_id FROM projects WHERE project_name='"+projectName+"'");
         if (rs.next())
         {
            //project already exist
            ret = false;
         }
         else
         {
            //add project to db
            rs.close();

            java.util.Date now = new java.util.Date();
            long i_now = (long) (now.getTime() / 1000);
            rs = st.executeQuery("INSERT INTO projects (project_name, project_owner, project_create_date) VALUES ('"+projectName+"', "+projectLeader+", "+i_now+")");
            ResultSet rs_fid = st.executeQuery("SELECT max(project_id) FROM projects");
            if (rs_fid.next())
            {
               int project_id = rs_fid.getInt(1);
               //assign users for project
               String inserted_users = "#"+projectLeader+"#";
               String user_id;
               st.execute("INSERT INTO users_projects (project_id, user_id) VALUES ("+project_id+", "+projectLeader+")");
               while (stok.hasMoreTokens())
               {
                  user_id = (String)stok.nextElement();
                  if (inserted_users.indexOf("#"+user_id+"#") == -1)
                  {
                     inserted_users = inserted_users + user_id + "#";
                     st.execute("INSERT INTO users_projects (project_id, user_id) VALUES ("+project_id+", "+user_id+")");
                  }
               }
               ret = true;
            }
         }
      }
      catch (Exception ex)
      {
         System.out.println("Error: newProject: "+ex.getMessage());
      }
      return(ret);
   }
}
